from TDhelper.generic.transformationType import transformation
from TDhelper.document.excel.FieldType import *
from types import FunctionType, MethodType, ModuleType
from openpyxl import load_workbook
import csv
import copy

class _AttributeOverride:
    def __init__(self, name, m_type):
        self._name = name
        self._type = m_type

    def __get__(self, instance, owen):
        return instance.__dict__[self._name]

    def __set__(self, instance, value):
        instance.__dict__[self._name] = transformation(value, self._type)

    def __delete__(self, instance):
        instance.__dict__.pop(self._name)

class Meta:
    file = None
    sheet = 'sheet1'
    extension = 'xlsx' 


class modelMeta(type):
    def __new__(cls, name, bases, dct):
        attrs = {'mapping': {}, 'Meta':Meta, '__exit__': __exit__, 'readLine': readLine,
                 'close': close, '__initExcelHandle__': __initExcelHandle__}
        for name, value in dct.items():
            if (not isinstance(dct[name], type)) and (not isinstance(dct[name], FunctionType)):
                if not name.startswith('__'):
                    if isinstance(dct[name], FieldType):
                        attrs['mapping'][name] = value.bindCol
                        attrs[name] = _AttributeOverride(name, value.fieldType)
                    else:
                        raise Exception('field type must is FieldType.')
            else:
                if isinstance(dct[name], type):
                    if name == 'Meta':
                        for attr_name in dct[name].__dict__:
                            if not attr_name.startswith('__'):
                                setattr(attrs['Meta'], attr_name, dct[name].__dict__[attr_name])
                else:
                    attrs[name] = value
        return super(modelMeta, cls).__new__(cls, name, bases, attrs)




def __initExcelHandle__(self):
    try:
        if self.Meta.file:
            m_extension= self.Meta.file.rsplit('.')[1]
            if m_extension== 'csv':
                self.Meta.extension= 'csv'
                self.__excelHandle__= open(self.Meta.file)
                self.__sheetHandle__= csv.reader(self.__excelHandle__)
            else:
                self.Meta.extension= 'xlsx'
                self.__excelHandle__ = load_workbook(self.Meta.file)
                self.__sheetHandle__ = self.__excelHandle__[self.Meta.sheet]
        else:
            raise Exception('meta file is None.')
    except Exception as e:
        raise e


def __exit__(self, exc_type, exc_value, exc_t):
    self.close()


def close(self):
    self.__excelHandle__ = None
    self.__sheetHandle__ = None


def readLine(self, lineOffset=1):
    if lineOffset < 1:
        lineOffset = 1
    if self.__sheetHandle__:
        if self.Meta.extension == 'xlsx':
            if lineOffset <= self.__sheetHandle__.max_row:
                rowdata = []
                column = self.__sheetHandle__.max_column+1
                for i in range(1, column):
                    cellvalue = self.__sheetHandle__.cell(
                        row=lineOffset, column=i).value
                    rowdata.append(cellvalue)
                for (name, value) in self.mapping.items():
                    if value <= len(rowdata):
                        setattr(self, name, rowdata[value-1])
                    else:
                        raise Exception('mapping error:(%s,%s)' % (name, value))
                return self
            else:
                return None
        else:
            rowdata= []
            for offset in range(0,lineOffset):
                try:
                    rowdata= next(self.__sheetHandle__)
                except Exception as e:
                    rowdata=[]
            if rowdata:
                for (name, value) in self.mapping.items():
                    if value <= len(rowdata):
                        attr_value=rowdata[value-1]
                        if attr_value=='None':
                            attr_value='0.00'
                        setattr(self, name, attr_value)
                    else:
                        raise Exception('mapping error:(%s,%s)' % (name, value))
                return self
            else:
                return None
    else:
        raise Exception('Sheet Handle is None.')



from multiprocessing import Process, Lock
from threading import Thread
import queue
import copy


class threadPools:
    def __init__(self, threadfunc, daemon=True, isJoin=False, maxSize=10):
        self.Daemon = daemon
        self.isJoin = isJoin
        self._processFlag = True
        self._threadLockArgs = Lock()
        self._threadLockThreadFunc = Lock()
        self._threadLockThreadFunc = Lock()
        self._activeThread = {}
        self._list = queue.Queue(maxSize)
        self._args = []
        for offset in range(0, maxSize):
            self._list.put((offset, threadfunc))

    def stop(self):
        self._processFlag = False

    def restart(self):
        self._processFlag = True

    def pushArgs(self, param: any):
        self._threadLockArgs.acquire()
        self._args.append(param)
        self._threadLockArgs.release()
        self.popThread()

    def popArgs(self):
        value = None
        self._threadLockArgs.acquire()
        if len(self._args) > 0:
            value = self._args[0]
            del self._args[0]
        self._threadLockArgs.release()
        return value

    def getArgsLength(self):
        value = 0
        self._threadLockArgs.acquire()
        value = len(self._args)
        self._threadLockArgs.release()
        return value

    def popThread(self):
        try:
            if not self._list.empty():
                self._threadLockThreadFunc.acquire()
                m_thread = self._list.get()
                self._threadLockThreadFunc.release()
                if m_thread:
                    self._activeThread[m_thread[0]] = m_thread[1]
                    m_args = self.popArgs()
                    if m_args:
                        m_thread = Thread(target=self._activeThread[m_thread[0]], name='Thread%s' % str(
                            m_thread[0]), args=(self, m_thread[0], m_args,))
                        m_thread.setDaemon = self.Daemon
                        m_thread.start()
                        if self.isJoin:
                            m_thread.join()
                else:
                    raise Exception('thread func is none.')
        except Exception as e:
            raise e

    def pushThread(self, m_thread: tuple):
        if not self._list.full():
            self._threadLockThreadFunc.acquire()
            self._list.put(m_thread)
            self._threadLockThreadFunc.release()
            if self.getArgsLength() > 0:
                self.popThread()
        else:
            raise Exception('thread pools is full.')

    def threadComplete(self, offset):
        m_thread = (offset, self._activeThread.pop(offset))
        self.pushThread(m_thread)






from TDhelper.document.excel.meta.modelMeta import modelMeta
class model(metaclass=modelMeta):
    __excelHandle__= None
    __sheetHandle__= None
    def __init__(self, excelPath= None):
        if excelPath:
            self.Meta.file= excelPath
        if self.Meta.file:
            self.__initExcelHandle__()

    def __initExcelHandle__(self):
        return None

    def __enter__(self):
        return self

    def readLine(self, lineOffset=1):
        return True

    def close(self):
        return None
    
    class Meta:
        file= ''
        sheet= 'sheet1'
        extension= 'xlsx'


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__autho__ = "Tony.Don"
__lastupdatetime__ = "2017/09/20"

import requests
from requests import exceptions
'''
class
    TDhelper.generic.m_http
description
    http帮助类
'''
class m_http:
    context = None
    res = None 
    def __init__(self):
        self.context = requests
        
        
    #url:wetsite url address
    def getcontent(self,p_url, p_timeout= 5, headers:dict= None):
        '''
        Featuren\r\n
            getcontent(self,url)\r\n
        Description\r\n
            获取url内容\r\n
        Args\r\n
            url\r\n
                type:string\r\n
                description:目标URL\r\n
        '''
        if self.context:
            try:
                if not headers:
                    self.res=self.context.get(url= p_url, timeout= p_timeout)
                else:
                    self.res= self.context.get(url= p_url, timeout= p_timeout, headers= headers)
                if self.res:
                    return self.res.text, self.res.status_code
                return p_url, "REQUESTS_OBJECT_IS_NULL"
            except exceptions.HTTPError as e:
                return e, "CONNECT_IS_ERROR"
            except exceptions.Timeout as e:
                return e,"TIME_OUT"
            except exceptions.ConnectTimeout as e:
                return e, 'CONNECT_TIME_TIMEOUT'
            except exceptions.ProxyError as e:
                return e,'PROXY_ERROR'
            except exceptions.SSLError as e:
                return e, 'SSL_ERROR'
            except exceptions.URLRequired as e:
                return e, 'URL_REQUIRED'
            except exceptions.BaseHTTPError as e:
                return e, 'BASE_HTTP_ERROR'
            except Exception as e:
                return e,"ERROR"

    def download(self,p_url, p_timeout= 5, headers:dict= None):
        if self.context:
            try:
                if not headers:
                    self.res=self.context.get(url= p_url, timeout= p_timeout)
                else:
                    self.res= self.context.get(url= p_url, timeout= p_timeout, headers= headers)
                if self.res:
                    return self.res.content, self.res.status_code
                return p_url, "REQUESTS_OBJECT_IS_NULL"
            except Exception as e:
                return e, "CONNECT_IS_ERROR"

    def https(self,url,*args,**kwargs):
        '''
        certificate
        '''
        pass